@@ -91,7 +91,7 @@ def pillar_update():
         known_hosts.update(env.host)
         local("scp -P {0} {1}bootmachine.sls {2}@{3}:/tmp/bootmachine.sls".format(env.port, pillar_dir, env.user, env.host))
     sudo("mv /tmp/bootmachine.sls /srv/pillar/bootmachine.sls")
-    sudo("salt '*' saltutil.refresh_pillar")
+    sudo("salt '*' saltutil.refresh_pillar &")  # background because it hangs on debian 6
 
 
 @task
@@ -113,7 +113,7 @@ def update_master_iptables():
         insert_line = sudo("iptables -L --line-numbers | grep {0}".format(configurator_ports[0]))
 
     if not insert_line:
-        print(yellow("iptables are wide open during first boot of a master"))
+        print(yellow("NOTE: iptables are wide open during first boot of a master"))
         return
 
     for port in configurator_ports:
@@ -127,18 +127,20 @@ def update_master_iptables():
 @task
 def launch():
     """
-    After salt is installed, accept the new minions, upload states,
-    and call highstate.
+    After the salt packages are installed, accept the new minions,
+    upload states, and call highstate.
     """
     if env.host != env.master_server.public_ip:
         abort("tried to launch on a non-master server")
 
-    local("fab master configurator.update_master_iptables")
     upload_saltstates()
 
+    # add ipaddresses from the newly booted servers to the pillar and update
+    local("fab master configurator.update_master_iptables")
+    pillar_update()
+
     time.sleep(10)  # sleep a little to give minions a chance to become visible
     accept_minions()
-    pillar_update()
 
     highstate()
 
@@ -164,7 +166,7 @@ def accept_minions():
                 sudo("salt-key --quiet --accept={0}".format(server))
         accepted = filter(None, sudo("salt-key --raw-out --list acc").translate(None, "'[]\ ").split(","))
         if len(accepted) != len(settings.SERVERS):
-            local("fab all configurator.restart")
+            local("fab each configurator.restart")
             time.sleep(5)
             slept += 5
             print(yellow("there are still unaccpeted keys, trying again."))
@@ -247,4 +249,4 @@ def start(distro):
     Start the salt master and minion daemons.
     Simply wrap salt's start method for the chosen distro.
     """
-    distro.setup_salt()
+    distro.start_salt()
