@@ -2,12 +2,15 @@
 BOOTMACHINE: A-Z transmutation of aluminium into rhodium.
 """
 import copy
+import getpass
+import logging
 import sys
 import telnetlib
 
 from fabric.api import env, local, task, run, sudo
 from fabric.decorators import parallel, task
 from fabric.colors import blue, cyan, green, magenta, red, white, yellow
+from fabric.contrib.files import exists
 from fabric.exceptions import NetworkError
 from fabric.network import connect
 from fabric.operations import reboot
@@ -45,64 +48,42 @@ configurator = import_module(settings.CONFIGURATOR_MODULE)
 @task(default=True)
 def bootmachine():
     """
-    Boot & provision all servers as per the config.
+    Boot, bootstrap and configure all servers as per the settings.
 
     Usage:
         fab bootmachine
     """
-    local_user = env.user
     # set environment variables
     master()
     env.new_server_booted = False
 
     # boot new servers in serial to avoid api overlimit
     boot()
-    print(green("all servers are booted."))
-    output = local("fab all ssh_test", capture=True)
+
+    output = local("fab each ssh_test", capture=True)
     if "CONFIGURATOR FAIL!" not in output:
-        print(green("all servers are booted, provisioned, and configured."))
+        print(green("all servers are fully provisioned."))
         return
 
-    # provision all servers in parallel for speed
-    local("fab all provision")
-    print(green("all servers are provisioned."))
+    # bootstrap all servers in parallel for speed
+    local("fab each bootstrap")
+    print(green("all servers are bootstrapped."))
 
-    # run the configurator in serial on the master to configure the new servers
+    # run the configurator from the the master server, maximum of 5x
     master()
-    for server in env.bootmachine_servers:
-        if server.port != int(settings.SSH_PORT):
-            configurator.launch()
-            launch_attempts = 1
-            break
-    print(green("the configurator has been run for all servers."))
+    __set_unconfigured_servers()
+    while env.unconfigured_servers and env.configure_attempts != 5:
+        print(env.unconfigured_servers, env.configure_attempts)
+        configure()
 
-    # ensure that the servers have been properly configured
-    env.user = local_user
-    master()
-    configured_servers = []
-    for server in env.bootmachine_servers:
-        while server.port != int(settings.SSH_PORT):
-            configurator.restart()
-            configurator.launch()
-            if server not in configured_servers:
-                configured_servers.append(server)
-            launch_attempts += 1
-            master()
-            if launch_attempts == 5:
-                abort("CONFIGURATOR FAIL!")
-
-    # in case a configurator change requires reboot, e.g. custom kernel
-    for server in configured_servers:
-        reboot_server(server)
-
-    # lastly, confirm that every server's SSH port matches settings.SSH_PORT
-    output = local("fab all ssh_test", capture=True)
+    # lastly, double check that the SSH settings for each server are correct
+    output = local("fab each ssh_test", capture=True)
     if "CONFIGURATOR FAIL!" in output:
         print(red("configurator failure."))
     else:
         # change the following output with caution
         # runtests.sh depends on exactly the following successful output
-        print(green("all servers are booted, provisioned, and configured."))
+        print(green("all servers are fully provisioned."))
 
 
 @task
@@ -121,31 +102,34 @@ def boot():
         server = servers.pop(0)
         if not server["servername"] in [server.name for server in env.bootmachine_servers]:
             provider.bootem(settings.SERVERS)
+            print(green("new server(s) have been booted."))
             env.new_server_booted = True
+            print(green("all servers are booted."))
             return
+    print(green("all servers are booted."))
 
 
 @task
 @parallel
-def provision():
+def bootstrap():
     """
-    Provision all unprovisioned servers.
-    Installs and start the configurator process.
+    Bootstraps the configurator on all newly booted servers.
+    Installs the configurator and starts its processes.
     Does not run the configurator.
 
     Usage:
-        fab all provision
+        fab each bootstrap
     """
     if int(settings.SSH_PORT) == 22:
-        abort("provision(): Security Error! Change ``settings.SSH_PORT`` to something other than ``22``")
+        abort("bootstrap(): Security Error! Change ``settings.SSH_PORT`` to something other than ``22``")
 
     __set_ssh_vars(env)
 
-    if env.port == int(settings.SSH_PORT):
-        print(green("{ip_addr} is already provisioned, skipping.".format(ip_addr=env.host)))
+    if exists("/root/.bootmachine_bootstrapped", use_sudo=True):
+        print(green("{ip_addr} is already bootstrapped, skipping.".format(ip_addr=env.host)))
         return
 
-    print(cyan("... {ip_addr} has started provisioning.".format(ip_addr=env.host)))
+    print(cyan("... {ip_addr} has begun bootstrapping .".format(ip_addr=env.host)))
 
     # upgrade distro
     server = [s for s in env.bootmachine_servers if s.public_ip == env.host][0]
@@ -157,6 +141,40 @@ def provision():
     configurator.setup(distro)
     configurator.start(distro)
     run("iptables -F")  # flush defaults before configuring
+    run("touch /root/.bootmachine_bootstrapped")
+
+    print(green("{0} is bootstrapped.".format(server.name)))
+
+
+@task
+def configure():
+    """
+    Configure all unconfigured servers.
+
+    Usage:
+        fab configure
+    """
+    master()
+    __set_unconfigured_servers()
+
+    # run the configurator and refresh the env variables by calling master()
+    if env.unconfigured_servers:
+        configurator.launch()
+        env.configure_attempts += 1
+        master()
+        # determine if configuration was a success and reboot just in case.
+        # for example, a reboot is required when rebuilding a custom kernel
+        for server in env.unconfigured_servers:
+            server = __set_ssh_vars(server)
+            if server.port == int(settings.SSH_PORT):
+                reboot_server(server.name)
+            else:
+                print(red("after {0} attempt, server {1} is still unconfigured".format(env.configure_attempts, server.name)))
+
+    __set_unconfigured_servers()
+    __set_ssh_vars(env)
+    if not env.unconfigured_servers:
+        print(green("all servers are configured."))
 
 
 @task
@@ -181,6 +199,7 @@ def reboot_server(name):
         env.user = "root"
     except IOError:
         env.port = int(settings.SSH_PORT)
+        env.user = getpass.getuser()
         telnetlib.Telnet(server.public_ip, env.port)
     env.host_string = "{0}:{1}".format(server.public_ip, env.port)
 
@@ -198,7 +217,7 @@ def ssh_test():
     Prove that ssh is open on `settings.SSH_PORT`.
 
     Usage:
-        fab all ssh_test
+        fab each ssh_test
     """
     __set_ssh_vars(env)
 
@@ -213,7 +232,7 @@ def ssh_test():
 
 
 @task
-def all():
+def each():
     """
     Set the env variables for a command to be run on all servers.
 
@@ -222,7 +241,6 @@ def all():
         currently does not support a multi-master configuration.
     """
     __shared_setup()
-
     for server in env.bootmachine_servers:
         if server.public_ip not in env.hosts:
             env.hosts.append(server.public_ip)
@@ -247,9 +265,10 @@ def master():
 
 def __shared_setup():
     """
-    Set the env variables common to both master() and all().
+    Set the env variables common to both master() and each().
     """
-    provider.get_ips()
+    provider.set_bootmachine_servers()
+
     for server in env.bootmachine_servers:
         if server.name == settings.MASTER:
             env.master_server = server
@@ -291,7 +310,22 @@ def __set_ssh_vars(valid_object):
         valid_object.user = "root"
     else:
         valid_object.configured = True
-        valid_object.user = env.user
+        valid_object.user = getpass.getuser()
 
     valid_object.host_string = "{0}:{1}".format(public_ip, port)
     return valid_object
+
+
+def __set_unconfigured_servers():
+    if not hasattr(env, "configure_attempts"):
+        env.configure_attempts = 0
+
+    env.unconfigured_servers = []
+    for server in env.bootmachine_servers:
+        if server.port != int(settings.SSH_PORT):
+            env.unconfigured_servers.append(server)
+
+
+# Resolve issue with paramiko which occasionaly causes bootmachine to hang
+# http://forum.magiksys.net/viewtopic.php?f=5&t=82
+logging.getLogger("paramiko").setLevel(logging.DEBUG)
